package mcmultipart.microblock;

import java.util.Collection;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Set;

import mcmultipart.multipart.MultipartRegistry;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.init.Blocks;

public class MicroblockRegistry {

    private static final Map<String, IMicroMaterial> materials = new LinkedHashMap<String, IMicroMaterial>();
    private static final Set<MicroblockClass> microClasses = new HashSet<MicroblockClass>();

    public static void registerMicroClass(MicroblockClass microClass) {

        microClasses.add(microClass);
        MultipartRegistry.registerProvider(microClass, microClass.getType());
    }

    public static <T extends IMicroMaterial> T registerMaterial(T material) {

        if (material == null) throw new NullPointerException("Attempting to register a null micro material!");
        String name = material.getName();
        if (materials.containsKey(name))
            throw new IllegalArgumentException("Attempting to register a micro material with a name that's already in use!");
        materials.put(name, material);
        return material;
    }

    public static IMicroMaterial registerMaterial(IBlockState blockState) {

        return registerMaterial(new BlockMicroMaterial(blockState));
    }

    public static IMicroMaterial registerMaterial(Block block, int meta) {

        return registerMaterial(block.func_176203_a(meta));
    }

    public static IMicroMaterial[] registerMaterial(Block block, int fromMeta, int toMeta) {

        IMicroMaterial[] materials = new IMicroMaterial[toMeta - fromMeta + 1];
        for (int i = fromMeta; i <= toMeta; i++)
            materials[i - fromMeta] = registerMaterial(block, i);
        return materials;
    }

    public static IMicroMaterial registerMaterial(Block block) {

        return registerMaterial(block.func_176223_P());
    }

    public static Collection<IMicroMaterial> getRegisteredMaterials() {

        return Collections.unmodifiableCollection(materials.values());
    }

    public static IMicroMaterial getMaterial(String name) {

        return materials.get(name);
    }

    static {
        registerMaterial(Blocks.field_150348_b, 0, 6);
        registerMaterial(Blocks.field_150347_e);
        registerMaterial(Blocks.field_150344_f, 0, 5);
        registerMaterial(Blocks.field_150368_y);
        registerMaterial(Blocks.field_150322_A, 0, 2);
        registerMaterial(Blocks.field_150325_L, 0, 15);
        registerMaterial(Blocks.field_150340_R);
        registerMaterial(Blocks.field_150339_S);
        registerMaterial(Blocks.field_150336_V);
        registerMaterial(Blocks.field_150342_X);
        registerMaterial(Blocks.field_150341_Y);
        registerMaterial(Blocks.field_150343_Z);
        registerMaterial(Blocks.field_150484_ah);
        // registerMaterial(Blocks.crafting_table);
        registerMaterial(Blocks.field_150423_aK);
        registerMaterial(Blocks.field_150424_aL);
        registerMaterial(Blocks.field_150425_aM);
        registerMaterial(Blocks.field_150417_aV, 0, 3);
        registerMaterial(Blocks.field_150385_bj);
        registerMaterial(Blocks.field_150377_bs);
        registerMaterial(Blocks.field_150475_bE);
        registerMaterial(Blocks.field_150371_ca, 0, 2);
        registerMaterial(Blocks.field_150406_ce, 0, 15);
        registerMaterial(Blocks.field_180397_cI, 0, 2);
        registerMaterial(Blocks.field_150407_cf);
        registerMaterial(Blocks.field_150405_ch);
        registerMaterial(Blocks.field_150402_ci);
        registerMaterial(Blocks.field_150432_aD);
        registerMaterial(Blocks.field_150403_cj);
        registerMaterial(Blocks.field_180395_cM, 0, 2);
        registerMaterial(Blocks.field_150359_w);
        registerMaterial(Blocks.field_150399_cn, 0, 15);
        registerMaterial(Blocks.field_150428_aP);
        registerMaterial(Blocks.field_150426_aN);
        registerMaterial(Blocks.field_180398_cJ);
        registerMaterial(Blocks.field_150451_bX);
    }

}
