package mcmultipart.raytrace;

import java.util.Collection;

import mcmultipart.multipart.IMultipart;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.entity.player.EntityPlayerMP;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.MovingObjectPosition;
import net.minecraft.util.Vec3;
import net.minecraft.world.World;

public final class RayTraceUtils {

    private static class RayTraceResultBase<T extends MovingObjectPosition> {

        public final AxisAlignedBB bounds;
        public final T hit;

        public RayTraceResultBase(T mop, AxisAlignedBB bounds) {

            this.hit = mop;
            this.bounds = bounds;
        }

        public boolean valid() {

            return hit != null && bounds != null;
        }

        public void setBounds(World world, BlockPos pos) {

            world.func_180495_p(pos)
                    .func_177230_c()
                    .func_149676_a((float) bounds.field_72340_a, (float) bounds.field_72338_b, (float) bounds.field_72339_c, (float) bounds.field_72336_d,
                            (float) bounds.field_72337_e, (float) bounds.field_72334_f);
        }

        public double squareDistanceTo(Vec3 vec) {

            return hit.field_72307_f.func_72436_e(vec);
        }
    }

    public static class RayTraceResult extends RayTraceResultBase<MovingObjectPosition> {

        public RayTraceResult(MovingObjectPosition mop, AxisAlignedBB bounds) {

            super(mop, bounds);
        }
    }

    public static class RayTraceResultPart extends RayTraceResultBase<PartMOP> {

        public RayTraceResultPart(RayTraceResult result, IMultipart part) {

            super(new PartMOP(result.hit, part), result.bounds);
        }

        public RayTraceResultPart(PartMOP mop, AxisAlignedBB bounds) {

            super(mop, bounds);
        }
    }

    private RayTraceUtils() {

    }

    public static Vec3 getStart(EntityPlayer player) {

        return new Vec3(player.field_70165_t, player.field_70163_u + player.func_70047_e(), player.field_70161_v);
    }

    public static Vec3 getEnd(EntityPlayer player) {

        double reachDistance = player instanceof EntityPlayerMP ? ((EntityPlayerMP) player).field_71134_c.getBlockReachDistance()
                : (player.field_71075_bZ.field_75098_d ? 5.0D : 4.5D);
        Vec3 lookVec = player.func_70040_Z();
        Vec3 start = getStart(player);
        return start.func_72441_c(lookVec.field_72450_a * reachDistance, lookVec.field_72448_b * reachDistance, lookVec.field_72449_c * reachDistance);
    }

    public static RayTraceResult collisionRayTrace(World world, BlockPos pos, EntityPlayer player, Collection<AxisAlignedBB> boxes) {

        return collisionRayTrace(world, pos, getStart(player), getEnd(player), boxes);
    }

    public static RayTraceResult collisionRayTrace(World world, BlockPos pos, EntityPlayer player, AxisAlignedBB aabb, int subHit,
            Object hitInfo) {

        return collisionRayTrace(pos, getStart(player), getEnd(player), aabb, subHit, hitInfo);
    }

    public static RayTraceResult collisionRayTrace(World world, BlockPos pos, Vec3 start, Vec3 end, Collection<AxisAlignedBB> boxes) {

        double minDistance = Double.POSITIVE_INFINITY;
        RayTraceResult hit = null;

        if (world == null || pos == null || start == null || end == null) System.out.println(world + " " + pos + " " + start + " " + end);

        int i = -1;
        for (AxisAlignedBB aabb : boxes) {
            RayTraceResult result = aabb == null ? null : collisionRayTrace(pos, start, end, aabb, i, null);
            if (result != null) {
                double d = result.squareDistanceTo(start);
                if (d < minDistance) {
                    minDistance = d;
                    hit = result;
                }
            }
            i++;
        }

        return hit;
    }

    public static RayTraceResult collisionRayTrace(BlockPos pos, Vec3 start, Vec3 end, AxisAlignedBB bounds, int subHit, Object hitInfo) {

        start = start.func_72441_c((-pos.func_177958_n()), (-pos.func_177956_o()), (-pos.func_177952_p()));
        end = end.func_72441_c((-pos.func_177958_n()), (-pos.func_177956_o()), (-pos.func_177952_p()));

        Vec3 vec3 = start.func_72429_b(end, bounds.field_72340_a);
        Vec3 vec31 = start.func_72429_b(end, bounds.field_72336_d);
        Vec3 vec32 = start.func_72435_c(end, bounds.field_72338_b);
        Vec3 vec33 = start.func_72435_c(end, bounds.field_72337_e);
        Vec3 vec34 = start.func_72434_d(end, bounds.field_72339_c);
        Vec3 vec35 = start.func_72434_d(end, bounds.field_72334_f);

        if (!isVecInsideYZBounds(bounds, vec3)) vec3 = null;
        if (!isVecInsideYZBounds(bounds, vec31)) vec31 = null;
        if (!isVecInsideXZBounds(bounds, vec32)) vec32 = null;
        if (!isVecInsideXZBounds(bounds, vec33)) vec33 = null;
        if (!isVecInsideXYBounds(bounds, vec34)) vec34 = null;
        if (!isVecInsideXYBounds(bounds, vec35)) vec35 = null;

        Vec3 vec36 = null;

        if (vec3 != null && (vec36 == null || start.func_72436_e(vec3) < start.func_72436_e(vec36))) vec36 = vec3;
        if (vec31 != null && (vec36 == null || start.func_72436_e(vec31) < start.func_72436_e(vec36))) vec36 = vec31;
        if (vec32 != null && (vec36 == null || start.func_72436_e(vec32) < start.func_72436_e(vec36))) vec36 = vec32;
        if (vec33 != null && (vec36 == null || start.func_72436_e(vec33) < start.func_72436_e(vec36))) vec36 = vec33;
        if (vec34 != null && (vec36 == null || start.func_72436_e(vec34) < start.func_72436_e(vec36))) vec36 = vec34;
        if (vec35 != null && (vec36 == null || start.func_72436_e(vec35) < start.func_72436_e(vec36))) vec36 = vec35;

        if (vec36 == null) {
            return null;
        } else {
            EnumFacing enumfacing = null;

            if (vec36 == vec3) enumfacing = EnumFacing.WEST;
            if (vec36 == vec31) enumfacing = EnumFacing.EAST;
            if (vec36 == vec32) enumfacing = EnumFacing.DOWN;
            if (vec36 == vec33) enumfacing = EnumFacing.UP;
            if (vec36 == vec34) enumfacing = EnumFacing.NORTH;
            if (vec36 == vec35) enumfacing = EnumFacing.SOUTH;

            MovingObjectPosition mop = new MovingObjectPosition(vec36.func_72441_c(pos.func_177958_n(), pos.func_177956_o(), pos.func_177952_p()), enumfacing, pos);
            mop.subHit = subHit;
            mop.hitInfo = hitInfo;
            return new RayTraceResult(mop, bounds);
        }
    }

    private static boolean isVecInsideYZBounds(AxisAlignedBB bounds, Vec3 vec) {

        return vec == null ? false : vec.field_72448_b >= bounds.field_72338_b && vec.field_72448_b <= bounds.field_72337_e && vec.field_72449_c >= bounds.field_72339_c
                && vec.field_72449_c <= bounds.field_72334_f;
    }

    private static boolean isVecInsideXZBounds(AxisAlignedBB bounds, Vec3 vec) {

        return vec == null ? false : vec.field_72450_a >= bounds.field_72340_a && vec.field_72450_a <= bounds.field_72336_d && vec.field_72449_c >= bounds.field_72339_c
                && vec.field_72449_c <= bounds.field_72334_f;
    }

    private static boolean isVecInsideXYBounds(AxisAlignedBB bounds, Vec3 vec) {

        return vec == null ? false : vec.field_72450_a >= bounds.field_72340_a && vec.field_72450_a <= bounds.field_72336_d && vec.field_72448_b >= bounds.field_72338_b
                && vec.field_72448_b <= bounds.field_72337_e;
    }
}